// cnewton.cpp: Complex root finding by Newton's Method
// Note: must start with a non-zero imaginary part
#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <limits>
using namespace std;

complex<double> f(const complex<double>& x) {
   return x*x + x + 2.0;
}

complex<double> df(const complex<double>& x) {
   return 2.0*x + 1.0;
}

complex<double> g(const complex<double>& x) {
   return x - f(x)/df(x);  // Newton's formula
}

int main(int argc, char* argv[]) {
   if (argc > 2) {
      complex<double> guess(atof(argv[1]), atof(argv[2]));
      double eps = numeric_limits<double>::epsilon();
      cout.precision(numeric_limits<double>::digits10+2);

      for (int i = 0; i < 50; ++i) {
         complex<double> next = g(guess);
         cout << i << ": " << next << endl;

         // Check for near zero just in case
         complex<double> fval = f(next);
         if (abs(fval) == 0) 
            break;

         // Check for FP numbers close enough (1-2 ulps)
         if (abs(next - guess) <= abs(guess)*eps)
            break;
         guess = next;
      }
   }
}

