// fsin.cpp: Calculates sin(x) as a double. x is
// reduced to the interval [-pi/2, pi/2]. Uses
// the first k terms of the Taylor's Series (more
// would add nothing). Precomputes the coefficients.
// Uses Horner's rule.

#include <iostream>
#include <limits>
#include "ieee.h"
#include "ulps.h"
using namespace std;

double pi = 3.14159265358979323846;
double pi1 = 3.1416015625;
double pi2 = -8.908910206761537356617e-6;
const int k = 9;

// The coefficients of the Taylor's series
double c[k] = {
   1.0,
   -0.16666666666666665052,
   0.83333333333331650314e-2,
   -0.19841269841201840457e-3,
   0.27557319210152756119e-5,
   -0.25052106798274584544e-7,
   0.16058936490371589114e-9,
   -0.76429178068910467734e-12,
   0.27204790957888846175e-14};

double mysine(double x) {
   int n = int(x/pi + 0.5*sign(x));
//   x = x - n*pi; 
   x = x - n*pi1; 
   x = x - n*pi2; 
   double y = x*x;

   // Compute c[0] + c[1]*y + ... + c[6]*y^6
   double sum = c[k-1];
   for (int i = k-2; i >= 0; --i)
      sum = sum*y + c[i];

   sum = x*sum;
   return (n%2) ? -sum : sum;
}

int main() {
   cout.precision(numeric_limits<double>::digits10);
   cout << mysine(0.0) << endl;
   cout << ulps(mysine(0.0), sin(0.0)) << endl;
   cout << mysine(pi/2.0) << endl;
   cout << ulps(mysine(pi/2.0), sin(pi/2.0)) << endl;
   cout << mysine(3.0*pi/2.0) << endl;
   cout << ulps(mysine(3.0*pi/2.0), sin(3.0*pi/2.0)) << endl;
   cout << mysine(22.0) << endl;
   cout << sin(22.0) << endl;
   cout << ulps(mysine(22.0), sin(22.0)) << endl;
}
