// fsin.cpp: Calculates sin(x) as a float. x is
// reduced to the interval [-pi/2, pi/2]. Uses
// the first 7 terms of the Taylor's Series (more
// would add nothing). Precomputes the coefficients
// uses Horner's rule.

#include <iostream>
#include <limits>
#include "ieee.h"
using namespace std;

double pi = 3.14159265358979323846;
double pi1 = 3.1416015625;
double pi2 = -8.908910206761537356617e-6;
const int k = 5;

// The CW coefficients
float c[k] = {
   1.0, -0.1666665668, 0.8333025139e-2, -0.1980741872e-3, 0.2601903036e-5};

float mysine(float x) {
   int n = int(x/pi + 0.5*sign(x));
   x = x - n*pi1;
   x = x - n*pi2;
   float y = x*x;

   // Compute c[0] + c[1]*y + ... + c[6]*y^6
   float sum = c[k-1];
   for (int i = k-2; i >= 0; --i)
      sum = sum*y + c[i];

   sum = x*sum;
   return (n%2) ? -sum : sum;
}

int main() {
   cout.precision(numeric_limits<float>::digits10);
   cout.setf(ios::scientific);
   cout << mysine(0.0f) << endl;
   cout << mysine(pi/2.0f) << endl;
   cout << mysine(float(pi)) << endl;
   cout << mysine(3.0f*pi/2.0f) << endl;
   cout << mysine(22.0f) << endl;
   cout << sin(22.0f) << endl;
   cout << mysine(10.0e8f) << endl;
   cout << sin(10.0e8f) << endl;
}

/* Output:

0.000000e+00
1.000000e+00
-8.742278e-08
-1.000000e+00
-8.851309e-03
-8.851309e-03
*/

