// endian.cpp: Reveals endian-ness
#include <iostream>
using namespace std;

union Overlay {
public:
    float x;
    int i;
    unsigned char bytes[4];
    struct {
        unsigned b31:1;
        unsigned b30:1;
        unsigned b29:1;
        unsigned b28:1;
        unsigned b27:1;
        unsigned b26:1;
        unsigned b25:1;
        unsigned b24:1;
        unsigned b23:1;
        unsigned b22:1;
        unsigned b21:1;
        unsigned b20:1;
        unsigned b19:1;
        unsigned b18:1;
        unsigned b17:1;
        unsigned b16:1;
        unsigned b15:1;
        unsigned b14:1;
        unsigned b13:1;
        unsigned b12:1;
        unsigned b11:1;
        unsigned b10:1;
        unsigned b9:1;
        unsigned b8:1;
        unsigned b7:1;
        unsigned b6:1;
        unsigned b5:1;
        unsigned b4:1;
        unsigned b3:1;
        unsigned b2:1;
        unsigned b1:1;
        unsigned b0:1;
    } bits;
};

void display(Overlay o) {
    cout << "As an int: " << o.i << endl;
    cout << "As bytes: " << hex;
    for (int i = 0; i < 4; ++i)
        cout << static_cast<unsigned int>(o.bytes[i]) << ' ';
    cout << dec << "\nAs bits: ";
    cout << o.bits.b31;
    cout << o.bits.b30;
    cout << o.bits.b29;
    cout << o.bits.b28;
    cout << o.bits.b27;
    cout << o.bits.b26;
    cout << o.bits.b25;
    cout << o.bits.b24;
    cout << o.bits.b23;
    cout << o.bits.b22;
    cout << o.bits.b21;
    cout << o.bits.b20;
    cout << o.bits.b19;
    cout << o.bits.b18;
    cout << o.bits.b17;
    cout << o.bits.b16;
    cout << o.bits.b15;
    cout << o.bits.b14;
    cout << o.bits.b13;
    cout << o.bits.b12;
    cout << o.bits.b11;
    cout << o.bits.b10;
    cout << o.bits.b9;
    cout << o.bits.b8;
    cout << o.bits.b7;
    cout << o.bits.b6;
    cout << o.bits.b5;
    cout << o.bits.b4;
    cout << o.bits.b3;
    cout << o.bits.b2;
    cout << o.bits.b1;
    cout << o.bits.b0;
    cout << "\n\n";
}

// Returns true if machine is Little-endian
bool isLittleEndian() {
   unsigned char x = 1;
   struct Bits {
      unsigned bit7:1;
      unsigned rest:7;
   } *p = reinterpret_cast<Bits*>(&x);
   return p->bit7;
}

int main() {
    Overlay o;
    o.i = 1;
    display(o);
    o.x = 6.5f;
    display(o);
    cout << boolalpha << "Is little endian? " << isLittleEndian() << "\n\n";

    // Inspect 6.5 as a double (little endian is backwards)
    double y = 6.5;
    unsigned long* p = reinterpret_cast<unsigned long*>(&y);
    unsigned long high_word = p[1]; // Switch 1 and 0 on a Mac
    unsigned long low_word = p[0];  //      "
    cout << "high word = " << hex << high_word << endl;
    cout << "low word = " << low_word << endl;
    unsigned char* p2 = static_cast<unsigned char*>(&y);
    cout << "Bytes: ";
    for (int i = 0; i < 8; ++i)
        cout << static_cast<unsigned int>(p2[i]) << ' ';
    cout << endl;

    // Extract exponent (logical bits 1-11)
    // First, shift sign bit off the left end, then shift to the right 21 bits to lose the mantissa
    int raw_exponent = (high_word << 1) >> 21;
    int true_exponent = raw_exponent - 1023;
    cout << "exponent = " << true_exponent << endl;
}

/* Output:
As an int: 1
As bytes: 1 0 0 0 
As bits: 10000000000000000000000000000000

As an int: 1087373312
As bytes: 0 0 d0 40 
As bits: 00000000000000000000101100000010

Is little endian? true

high word = 401a0000
low word = 0
Bytes: 0 0 0 0 0 0 1a 40
exponent = 2
*/
