// fsin.cpp: Calculates sin(x) as a float. x is
// reduced to the interval [-pi/2, pi/2]. Uses
// the first 7 terms of the Taylor's Series (more
// would add nothing). Precomputes the coefficients
// uses Horner's rule.

#include <iostream>
#include <limits>
#include "../ieee.h"
using namespace std;

float pi = 3.14159265358979323846f;
const int k = 6;

// The coefficients of the Taylor's series
float c[k];

void coeff() {
   float den = 1.0f;
   int fact = 1;  // The denominator contains fact!
   c[0] = den;

   for (int i = 1; i < k; ++i) {
      den *= ++fact;
      den *= ++fact;
      c[i] = 1.0f/den;
      if (i%2)
         c[i] = -c[i];
   }
}

float mysine(float x) {
   int n = int(x/pi + 0.5*sign(x)); // use double precision here
   x = x - n*pi;
   float y = x*x;

   // Compute c[0] + c[1]*y + ... + c[5]*y^5
   float sum = c[k-1];
   for (int i = k-2; i >= 0; --i)
      sum = sum*y + c[i];

   sum = x*sum;
   return (n%2) ? -sum : sum;
}

int main() {
   coeff();
   cout << mysine(0.0f) << " (" << sin(0.0f) << ")\n";
   cout << mysine(pi/2.0f) << " (" << sin(pi/2.0f) << ")\n";
   cout << mysine(pi) << " (" << sin(pi) << ")\n";
   cout << mysine(3.0f*pi/2.0f) << " (" << sin(3.0f*pi/2.0f) << ")\n";
   cout << mysine(22.0f) << " (" << sin(22.0f) << ")\n";
   cout << mysine(10.0e8f) << " (" << sin(10.0e8f) << ")\n";
}
