// quad2a.cpp (uses single precision)
// Solves the equation x^2 + bx + c = 0 (i.e., a == 1)
// This version avoids potential catastrophic cancellation
// and computes r2 as r1/c.

#include <cfloat>
#include <cmath>
#include <iostream>
using namespace std;

int main() {
   cout.precision(FLT_DIG-1);
   cout.setf(ios::showpoint);
   cout.setf(ios::scientific);
   float b = -1.0e5f;
   float c = 1.0f;

   float d = b*b - 4.0f*c;
   if (d >= 0) {
      // Real roots
      float e = sqrt(d), r1;
      if (b >= 0)
         r1 = (-b - e) / 2.0f;
      else
         r1 = (-b + e) / 2.0f;
      float r2 = c/r1;
      cout << "r1 = " << r1 << ", r2 = " << r2 << endl;
   }
   else {
      // Complex roots
      float p = -b / 2.0f;
      float q = sqrt(-d) / 2.0f;
      cout << "r1 = " << p << " + " << q << "i\n";
      cout << "r2 = " << p << " - " << q << "i\n";
   }
}

/* Output:
r1 = 1.00000e+05, r2 = 1.00000e-05
*/
