// simpson.cpp: Calculate an area by Simpson's rule
#include <algorithm>
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <limits>
using namespace std;

double g(double x) {
   return 1.0/(1.0+x*x*x);
}

double simpson(double a, double b, double f(double), int n) {

   // Calculate sum with n panels
   assert(n%2 == 0); // n must be even
   double h = (b-a)/n;
   double x = a;
   double sum = f(a)+f(b);
   double factor[] = {2.0, 4.0};
   for (int i = 1; i <= n-1; ++i) {
      x += h;
      sum += factor[i%2]*f(x);
   }
   return sum * h/3.0;
}

int main(int argc, char* argv[]) {
   if (argc > 2) {
      int n = atoi(argv[1]);
      double a1 = simpson(0.0, 1.0, g, n);
      n *= 2;
      double a2 = simpson(0.0, 1.0, g, n);
      double eps = numeric_limits<double>::epsilon();
      double tol = atof(argv[2]);
      cout.precision(numeric_limits<double>::digits10+2);
      while (abs(a1 - a2) > max(tol, eps*abs(a2))) {
         a1 = a2;
         n *= 2;
         a2 = simpson(0.0, 1.0, g, n);
         cout << n << ": " << a2 << endl;
      }
   }
}

/* Output (roundoff takes over after this):
C:\UVSC\3320> simpson 2 .000000000000001
8: 0.83565941275841027
16: 0.83564951422702138
32: 0.83564888996350362
64: 0.83564885087204432
128: 0.8356488484276966
256: 0.83564884827490671
512: 0.83564884826535801
1024: 0.83564884826475971
2048: 0.83564884826472274
4096: 0.83564884826472541
8192: 0.83564884826471764
16384: 0.83564884826472186
32768: 0.8356488482647263
65536: 0.83564884826472052
131072: 0.83564884826472152
*/

